import { world, system, ItemStack, EnchantmentType, EntityComponentTypes, EquipmentSlot } from "@minecraft/server";
import * as game from "./game";
import * as Vector3 from "./vector3";
import * as heads from "./heads";
import * as inventory from "./inventory";


let passiveEffectsCD = 1;
let cooldownDisplayCD = 1;
export async function onTick()
{
	let players = await world.getAllPlayers();
	for (const player of players) 
	{
		if (player === undefined)
			continue;

		updatePlayerActionCooldowns(player);
		await updatePlayerMobHead(player);

		let helmetId = inventory.getHelmet(player);
		if (helmetId === "tcs_mbh:goat_head")
		{
			let protectFromFall = await game.verifyHeightAboveGround(player.location, 3);
			if (protectFromFall === true)
				player.addEffect("slow_falling", 40, {amplifier:1, showParticles:false});
		}
	}
	let horses = await game.getEntitiesWithTags("minecraft:horse", ["tcs_mbh_mount"]);
	for (const horse of horses)
	{
		let rideable = horse.getComponent("minecraft:rideable");
		let horseRides = rideable.getRiders();
		if (horseRides === undefined || horseRides.length === 0)
			{
				await game.playSound(horse, "@a", "tcs_mbh:horse_dismount");
				horse.remove();
			}
	}
	let snifferSpirits = await game.getEntitiesWithTags("tcs_mbh:sniffer_spirit");
	for (const snifferSpirit of snifferSpirits)
	{
		game.runMobCommand(snifferSpirit, "fill ~-1 ~ ~-1 ~1 ~3 ~1 air destroy");
		let mineCD = snifferSpirit.getDynamicProperty("tcs_mbh_mine_cd");
		mineCD = mineCD === undefined ? 10 : mineCD - 1;
		if (mineCD == 0)
		{
			
			let impulse = snifferSpirit.getDynamicProperty("tcs_mbh_impulse");
			snifferSpirit.applyImpulse(impulse);
			mineCD = 10;
		}
		snifferSpirit.setDynamicProperty("tcs_mbh_mine_cd", mineCD);
		let origin = snifferSpirit.getDynamicProperty("tcs_mbh_origin");
		if (Vector3.distance(origin, snifferSpirit.location) > 15)
			snifferSpirit.triggerEvent("tcs_mbh:despawn");
	}

	passiveEffectsCD--;
	if (passiveEffectsCD <= -1)
		passiveEffectsCD = 200;
	cooldownDisplayCD--;
	if (cooldownDisplayCD <= -1)
		cooldownDisplayCD = 20;
}

function updatePlayerActionCooldowns(player)
{
	let helmetId = inventory.getHelmet(player);
	cooldownDisplayCD;
	for (let itemData of heads.itemActionData)
	{
		let itemCDProperty = "tcs_mbh_cd_" + itemData.id.replace("tcs_mbh:","");
		let cd = player.getDynamicProperty(itemCDProperty);
		if (cd === undefined)
			continue;
		cd = (cd > 2) ? cd - 1 : undefined;
		player.setDynamicProperty(itemCDProperty, cd);

		if (itemData.id === helmetId)
		{
			if (cd > 20)
				game.actionbarQuick(player, "@s", `§6Special Ability Cooldown§f: ${Math.floor(cd/1200)}:${Math.floor((cd%1200)/20).toString().padStart(2, '0')}` );
			else if (cd === 2)
				game.actionbarQuick(player, "@s", `§6Special Ability§f: §aReady§f` );
			
		}
	}

	
	let mobHeadActionData = heads.getItemActionData(helmetId);
	if (mobHeadActionData !== undefined)
	{

	}
}
function getPlayerActionCooldown(player, itemActionData)
{
	let itemCDProperty = "tcs_mbh_cd_" + itemActionData.id.replace("tcs_mbh:","");
	return player.getDynamicProperty(itemCDProperty);
}
function setPlayerActionCooldown(player, itemActionData)
{
	let itemCDProperty = "tcs_mbh_cd_" + itemActionData.id.replace("tcs_mbh:","");
	player.setDynamicProperty(itemCDProperty, (itemActionData.cd * 20));
}

export async function updatePlayerMobHead(player)
{
	let helmetId = inventory.getHelmet(player);
	let previousHelmetId = player.getDynamicProperty("tcs_mbh_active_mob_head");
	let pandaRoll = player.getDynamicProperty("tcs_mbh_panda_roll");
	
	if ((helmetId === previousHelmetId) || (pandaRoll == true))
	{
		let mobHeadPassiveData = heads.getItemPassiveData(previousHelmetId);
		if (mobHeadPassiveData !== undefined && mobHeadPassiveData.effect !== undefined && mobHeadPassiveData.effect.id === "tcs_mbh:flight")
		{
			await handleFlight(player);
		}
		if (passiveEffectsCD === 0)
			await updatePassiveEffects(player, previousHelmetId, false);
		else  if (passiveEffectsCD % 20 === 0)
		{
			await updateWaterPassiveEffects(player, previousHelmetId, false);
			await updatePassiveAreaEffects(player, previousHelmetId, false);
		}
		return;
	}

	// handle effects from previous helmet
	if (previousHelmetId !== undefined)
		await handleRemovingMobHead(player, previousHelmetId);

	// handle effects from new helmet
	if (helmetId !== undefined)
		await handleWearingMobHead(player, helmetId);

}

async function updatePassiveEffects(player, helmetId, onlyRemove)
{
	let mobHeadPassiveData = heads.getItemPassiveData(helmetId);
	if (mobHeadPassiveData === undefined)
		return;
	if (mobHeadPassiveData.effect !== undefined)
		await updatePassiveEffect(player, mobHeadPassiveData.effect, onlyRemove);
	if (mobHeadPassiveData.effect2 !== undefined)
		await updatePassiveEffect(player, mobHeadPassiveData.effect2, onlyRemove);
	if (mobHeadPassiveData.effect3 !== undefined)
		await updatePassiveEffect(player, mobHeadPassiveData.effect3, onlyRemove);
}
async function updateWaterPassiveEffects(player, helmetId)
{
	let mobHeadPassiveData = heads.getItemPassiveData(helmetId);
	if (mobHeadPassiveData === undefined)
		return;
	if (mobHeadPassiveData.effect !== undefined && mobHeadPassiveData.effect.inWater === true)
		await updatePassiveEffect(player, mobHeadPassiveData.effect, false);
	if (mobHeadPassiveData.effect2 !== undefined && mobHeadPassiveData.effect2.inWater === true)
		await updatePassiveEffect(player, mobHeadPassiveData.effect2, false);
	if (mobHeadPassiveData.effect3 !== undefined && mobHeadPassiveData.effect3.inWater === true)
		await updatePassiveEffect(player, mobHeadPassiveData.effect3, false);
}
async function updatePassiveAreaEffects(player, helmetId)
{
	let mobHeadPassiveData = heads.getItemPassiveData(helmetId);
	if (mobHeadPassiveData === undefined)
		return;
	if (mobHeadPassiveData.areaEffect !== undefined)
		await addAreaEffect(player, mobHeadPassiveData.areaEffect);
}
async function updatePassiveEffect(player, passiveEffectData, onlyRemove)
{
	if (passiveEffectData.id.startsWith("tcs_mbh"))
		return;
	if (passiveEffectData.inWater === true && !player.isInWater)
		onlyRemove = true;
	if (onlyRemove === true)
	{
		player.removeEffect(passiveEffectData.id);
		return;
	}
	
	let level = passiveEffectData.level === undefined ? 0 : passiveEffectData.level-1;
	if (onlyRemove === false)
		player.addEffect(passiveEffectData.id, 400, {amplifier:level, showParticles:false});
}
async function removePassiveEffect(player, passiveEffectData)
{
	if (passiveEffectData.id.startsWith("tcs_mbh"))
		return;
	player.removeEffect(passiveEffectData.id);
}

async function handleFlight(player)
{
	if (!player.isJumping && player.isOnGround) // player landed on ground
	{
		await cancelFlight(player);
		return;
	}
	
	if (player.isJumping) // flying up
	{
		game.runPlayerCommand(player, "effect @s levitation 1 6 true");
	}
	else if(!player.isOnGround) // in air falling 
	{
		game.runPlayerCommand(player, "effect @s slow_falling 1 3 true");
	}
}
async function cancelFlight(player)
{
	game.runPlayerCommand(player, "effect @s slow_falling 0");
	game.runPlayerCommand(player, "effect @s levitation 0");
	//player.removeTag("tcs_mbh_flying");
}

async function updateActionEffect(player, effectData)
{
	if (effectData.id.startsWith("tcs_mbh"))
		return;
	//player.removeEffect(effectData.id);
	let level = effectData.level === undefined ? 0 : effectData.level-1;
	let ticks = effectData.time === undefined ? 40 : effectData.time * 20;
	player.addEffect(effectData.id, ticks, {amplifier:level, showParticles:false});
}
async function updateHitEffect(player, target, effectData)
{
	if (effectData.id.startsWith("tcs_mbh"))
		return;
	let level = effectData.level === undefined ? 0 : effectData.level-1;
	let ticks = effectData.time === undefined ? 40 : effectData.time * 20;
	if (effectData.id === "knockback")
	{
		let knockbackDirection = Vector3.getDirection(player.location, target.location);
		knockbackDirection.y = 0.5;
		target.applyImpulse(Vector3.copy(knockbackDirection).multiply(effectData.level).toLocation());
		return;
	}
	target.addEffect(effectData.id, ticks, {amplifier:level, showParticles:true});
}
async function handleRemovingMobHead(player, helmetId)
{
	player.setDynamicProperty("tcs_mbh_active_mob_head", undefined);
	let itemPassiveData = heads.getItemPassiveData(helmetId);
	if (itemPassiveData === undefined)
		return;
	await updatePassiveEffects(player, helmetId, true);
	if (itemPassiveData !== undefined && itemPassiveData.effect !== undefined && itemPassiveData.effect.id === "tcs_mbh:flight")
		cancelFlight(player);

	/// TODO - particles and others when removing helm
}
async function handleWearingMobHead(player, helmetId)
{
	let mobHeadPassiveData = heads.getItemPassiveData(helmetId);
	if (mobHeadPassiveData === undefined)
		return;

	const frontLocation = {
		x: player.location.x + player.getViewDirection().x * 1.5,
		y: player.location.y + 0.7,
		z: player.location.z + player.getViewDirection().z * 1.5
	}

	player.setDynamicProperty("tcs_mbh_active_mob_head", helmetId);
	player.dimension.spawnParticle(`${helmetId}_equip_icon`, frontLocation);
	player.dimension.spawnParticle(`${helmetId}_equip_ring`, player.location);

	let mobData = heads.getMobData(helmetId);
	//if (mobData.id === "minecraft:husk")
	//{
		//let text = mobData.display + "\n§6Effect§f: Inflicts §eHunger II§f (0:03) effect to any entity you hit.\n" +
		//		"§6Special Ability§f: Summons §e4 friendly baby husks§f that fight for you.\n" +
		//		"====================\nUse any sword and press §cINTERACT§f to activate special ability.";
		//game.tellraw(player, "@s", text);
	//}
	
	game.actionbar(player, "@s", `${mobData.display}: Equipped`)

	await updatePassiveEffects(player, helmetId, false);
	game.playSound(player, "@s", "tcs_mbh:equip_head");
	
	await game.runPlayerCommand(player, "camera @s set minecraft:third_person_front default");
	system.runTimeout(() => { releaseCamera(player); }, 25);
	/// TODO - particles and others when wearing helm
}
export async function releaseCamera(player)
{
	await game.runPlayerCommand(player, "camera @s clear");
}


const weaponsForAction = [ "minecraft:wooden_sword", "minecraft:stone_sword", "minecraft:iron_sword", "minecraft:golden_sword",
					"minecraft:diamond_sword", "minecraft:netherite_sword" ];

// Handle using mob amulet (requires player sneaking, and wearing mob head)
export async function onItemUse(event)
{
	let player = event.source;
	let itemId = event.itemStack.typeId;
	if (!weaponsForAction.includes(itemId))
		return;

	let helmetId = inventory.getHelmet(player);
	let mobHeadActionData = heads.getItemActionData(helmetId);
	if (mobHeadActionData === undefined)
		return;
	// check cooldown
	if (getPlayerActionCooldown(player, mobHeadActionData) !== undefined)
		return;
	
	// drain mob_head durability for use
	if (mobHeadActionData.useCost > 0)
		inventory.drainMobHeadDurability(player, mobHeadActionData);
	// set cooldown
	setPlayerActionCooldown(player, mobHeadActionData);

	game.playSound(player, "@a", `${helmetId}_ability`);

	if (mobHeadActionData.effect !== undefined)
		await updateActionEffect(player, mobHeadActionData.effect);
	if (mobHeadActionData.effect2 !== undefined)
		await updateActionEffect(player, mobHeadActionData.effect2);
	if (mobHeadActionData.effect3 !== undefined)
		await updateActionEffect(player, mobHeadActionData.effect3);
	if (mobHeadActionData.projectile !== undefined)
		await shootProjectile(player, helmetId, mobHeadActionData.projectile);

	if (mobHeadActionData.action !== undefined)
		await performAction(player, mobHeadActionData.action, mobHeadActionData);
	if (mobHeadActionData.areaEffect !== undefined)
		await addAreaEffect(player, mobHeadActionData.areaEffect);
	if (mobHeadActionData.summon !== undefined)
		await performSummon(player, mobHeadActionData.summon);
	// effect, action, particles, etc

	if (mobHeadActionData.particle !== undefined)
		player.dimension.spawnParticle(mobHeadActionData.particle, player.location);
	if (mobHeadActionData.particle2 !== undefined)
		player.dimension.spawnParticle(mobHeadActionData.particle2, player.location);
	if (mobHeadActionData.particle3 !== undefined)
		player.dimension.spawnParticle(mobHeadActionData.particle3, player.location);

	if (helmetId === "tcs_mbh:evoker_head")
		player.playAnimation("animation.mob_head.evoker_cast");
	if (helmetId === "tcs_mbh:panda_head"){//!
		//player.playAnimation("animation.mob_head.panda_rolling");
		
		const equippable = player.getComponent(EntityComponentTypes.Equippable);
		if (!equippable) return;

		//Save
		player.setDynamicProperty("tcs_mbh_panda_roll", true);
		const savedEquipment = {
			head: equippable.getEquipment(EquipmentSlot.Head) ?? new ItemStack("minecraft:air"),
			chest: equippable.getEquipment(EquipmentSlot.Chest) ?? new ItemStack("minecraft:air"),
			legs: equippable.getEquipment(EquipmentSlot.Legs) ?? new ItemStack("minecraft:air"),
			feet: equippable.getEquipment(EquipmentSlot.Feet) ?? new ItemStack("minecraft:air"),
			mainhand: equippable.getEquipment(EquipmentSlot.Mainhand) ?? new ItemStack("minecraft:air"),
			offhand: equippable.getEquipment(EquipmentSlot.Offhand) ?? new ItemStack("minecraft:air"),
		};
		//game.debug(`savedEquipment: ${JSON.stringify(savedEquipment)}`);
		
		//Remove
		equippable.setEquipment(EquipmentSlot.Head, new ItemStack("minecraft:air"));
		equippable.setEquipment(EquipmentSlot.Chest, new ItemStack("minecraft:air"));
		equippable.setEquipment(EquipmentSlot.Legs, new ItemStack("minecraft:air"));
		equippable.setEquipment(EquipmentSlot.Feet, new ItemStack("minecraft:air"));
		equippable.setEquipment(EquipmentSlot.Mainhand, new ItemStack("minecraft:air"));
		equippable.setEquipment(EquipmentSlot.Offhand, new ItemStack("minecraft:air"));

		equippable.setEquipment(EquipmentSlot.Head, new ItemStack("tcs_mbh:panda_transform_head")); //minecraft:iron_helmet

		player.addEffect("invisibility", 20, {amplifier:1, showParticles:false});
		await game.runPlayerCommand(player, "camera @s set minecraft:third_person default");
		let dir = player.getViewDirection();
		player.applyKnockback(dir.x, dir.z, Math.sqrt(dir.x ** 2 + dir.z ** 2)*8, dir.y*6);

        system.runTimeout(() => {
            equippable.setEquipment(EquipmentSlot.Head, savedEquipment.head);
            equippable.setEquipment(EquipmentSlot.Chest, savedEquipment.chest);
            equippable.setEquipment(EquipmentSlot.Legs, savedEquipment.legs);
            equippable.setEquipment(EquipmentSlot.Feet, savedEquipment.feet);
            equippable.setEquipment(EquipmentSlot.Mainhand, savedEquipment.mainhand);
            equippable.setEquipment(EquipmentSlot.Offhand, savedEquipment.offhand);
			releaseCamera(player);
			player.setDynamicProperty("tcs_mbh_panda_roll", false);
        }, 20);
	}
}

async function shootProjectile(player, helmetId, projectileId) {
	

	let shootAngles = [ 0 ];
	if (projectileId === "tcs_mbh:ghast_fireball")
		shootAngles = [ 15, 0, -15 ];

	let delay = 1;
	for (const shootAngle of shootAngles)
	{
		game.playSound(player, "@a", `${helmetId.replace("_head","_projectile_shoot")}`);
		system.runTimeout(() => { shootDelayedProjectile(player, projectileId, shootAngle); }, delay);
		delay += 10;
	}
}
async function shootDelayedProjectile(player, projectileId, shootAngle)
{
	/// Still working on this line for laser
	let Velocity;
	if(projectileId === "tcs_mbh:guardian_laser"){Velocity = 2;}//2.5
	else if(projectileId === "tcs_mbh:tp_orb"){Velocity = 3;}
	else Velocity = 1.8;//1
	const horizontalImpulse = projectileId === "tcs_mbh:sniffer_spirit" ? true : false;
	const direction = Vector3.fromRotation(player.getRotation(), shootAngle, horizontalImpulse);
	const spawnDirection = Vector3.fromRotation(player.getRotation(), shootAngle, horizontalImpulse).multiply(3);;
	const projectileVelocity = direction.multiply(Velocity);

	let playerLocation = new Vector3.Vector3(player.location);
	let spawnVerticalOffset = projectileId === "tcs_mbh:sniffer_spirit" ? 0 : 1.5;
	let projectileSpawn = playerLocation.add(Vector3.Up.multiply(spawnVerticalOffset)).add(spawnDirection);

	const projectile =  player.dimension.spawnEntity(projectileId, projectileSpawn.toLocation());
	let projectileComponent = projectile.getComponent("minecraft:projectile");
	projectileComponent.owner = player;
	projectile.applyImpulse(projectileVelocity.toLocation());
	
	if (projectileId === "tcs_mbh:sniffer_spirit")
	{
		projectile.setDynamicProperty("tcs_mbh_impulse", direction.toLocation());
		projectile.setDynamicProperty("tcs_mbh_origin", projectileSpawn.toLocation());
	}

	if (projectileId === "tcs_mbh:guardian_laser")
	{
		for(let dist = 0; dist< 20; dist+= 0.1)
			player.dimension.spawnParticle("tcs_mbh:guardian_laser", projectileSpawn.add(direction.multiply(dist)));
	
	}
}

async function performAction(player, actionData, mobHeadActionData) {
	if (actionData.id === "tcs_mbh:phantom_boost")
	{
		let dir = player.getViewDirection();
		player.applyKnockback(dir.x, dir.z, Math.sqrt(dir.x ** 2 + dir.z ** 2)*8, dir.y*6);
	}
	if (actionData.id === "tcs_mbh:creeper_blast")
	{
		game.playSound(player, "@a", "tcs_mbh:creeper_blast_charge");
		system.runTimeout(() => { delayedCreeperBlast(player, mobHeadActionData); }, 40);
	}
}
export async function delayedCreeperBlast(player, mobHeadActionData)
{
	game.playSound(player, "@a", "tcs_mbh:creeper_blast");
	addAreaEffect(player, { id: "knockback", radius: 4, level: 1, damage: 3});
	if (mobHeadActionData.delayedParticle !== undefined)
		player.dimension.spawnParticle(mobHeadActionData.delayedParticle, player.location);
}
async function addAreaEffect(player, areaEffectData) {
	
	let nearbyEntities = await game.getAllEntitiesInRange(player.dimension, player.location, 0.1, areaEffectData.radius);
	for (let nearbyEntity of nearbyEntities)
	{
		if (nearbyEntity.typeId === "tcs_mbh:book")
			continue;
		if (areaEffectData.id === "knockback")
		{
			let knockbackDirection = Vector3.getDirection(player.location, nearbyEntity.location);
			knockbackDirection.y += 0.2;
			nearbyEntity.applyImpulse(Vector3.copy(knockbackDirection).multiply(areaEffectData.level).toLocation());
			if (areaEffectData.damage !== undefined)
				nearbyEntity.applyDamage(areaEffectData.damage);
		}
		else if (areaEffectData.id === "fire")
			nearbyEntity.setOnFire(areaEffectData.time, true);
		else
			nearbyEntity.addEffect(areaEffectData.id, areaEffectData.time*20, {amplifier:areaEffectData.level-1, showParticles:true});
	}
}
async function performSummon(player, summonData) {
	
	let summonId = summonData.id;
	let summonCount = summonData.count === undefined ? 1 : summonData.count;
	let summonAngle = [ 0 ];
	if (summonCount === 3)
		summonAngle = [ 15, 0, -15 ];
	else if (summonCount === 4)
		summonAngle = [ 20, 7, -7, -20 ];
	for (let c=0; c<summonCount; c++)
	{
		
		const direction = Vector3.fromRotation(player.getRotation(), summonAngle[c], true);
		const spawnLocation = direction.multiply(3).add(player.location).toLocation();
		if (summonId === "tcs_mbh:vex")
			spawnLocation.y += 1;

		let spawnedMob = player.dimension.spawnEntity(summonId, spawnLocation);
		if (summonId === "tcs_mbh:vex")
			player.dimension.spawnParticle("tcs_mbh:vex_spawn", spawnLocation);
		if (summonId === "tcs_mbh:husk")
			player.dimension.spawnParticle("tcs_mbh:husk_spawn", spawnLocation);
		if (summonId === "horse")
		{
			await spawnedMob.triggerEvent("minecraft:ageable_grow_up");
			await spawnedMob.triggerEvent("minecraft:on_tame");
			await spawnedMob.triggerEvent("minecraft:horse_saddled");
			let rideable = spawnedMob.getComponent("minecraft:rideable");
			rideable.addRider(player);
			spawnedMob.addTag("tcs_mbh_mount");
		}
		else if (summonData.tame === true)
		{
			let tameable = spawnedMob.getComponent("minecraft:tameable");
			let tamemount = spawnedMob.getComponent("minecrarft:tamemount");
			if (tameable !== undefined)
				tameable.tame(player);
			if (tamemount !== undefined)
				tamemount.tame();
		}
	}
}

// Handle mob head drop on mob kill
export async function onEntityDie(event) {
	let entityId = event.deadEntity.typeId;
	if (entityId.startsWith("tcs_mbh:"))
		return;

	//event.damageSource.damagingEntity.typeId === "minecraft:player"
	let mobData = heads.getMobData(entityId);

	let entityDimension = undefined;
	let entityLocation = undefined;
	if (entityId === "minecraft:wither") {
		entityDimension = world.getDimension(game.host.getDynamicProperty("tcs_mbh_wither_dimension"));
		entityLocation = game.host.getDynamicProperty("tcs_mbh_wither_location");
	}
	else {
		entityDimension = event.deadEntity.dimension;
		entityLocation = event.deadEntity.location;
	}

	if (mobData !== undefined)
	{
		let drop_rng = Math.floor(Math.random() * 100) + 1;
		if (drop_rng <= mobData.dropRate)
		{
			let headItem = new ItemStack(mobData.item, 1);
			if (mobData.item === "tcs_mbh:creeper_head")
			{
				const enchantableComponent = headItem.getComponent("minecraft:enchantable");
				let enchantmentType = new EnchantmentType("blast_protection");
				enchantableComponent.addEnchantment({ type: enchantmentType, level: 2 });
			}
			entityDimension.spawnItem(headItem, entityLocation);
		}
	}

	const damageSource = event.damageSource;
	const damagingEntity = damageSource === undefined ? undefined : damageSource.damagingEntity;
	const attackerMobHeadPassiveData = heads.getItemPassiveData(inventory.getHelmet(damagingEntity));
	if (attackerMobHeadPassiveData !== undefined && attackerMobHeadPassiveData.effect !== undefined)
	{
		if (attackerMobHeadPassiveData.effect.id === "tcs_mbh:loot")
		{
			let drop_rng = Math.floor(Math.random() * 100) + 1;
			if (drop_rng <= attackerMobHeadPassiveData.effect.chance)
			{
				entityDimension.spawnItem(new ItemStack(attackerMobHeadPassiveData.effect.item, 1), entityLocation);
				game.playSound(damagingEntity, "@a", "tcs_mbh:loot_drop");
			}
		}
	}
}

export async function onEntityHurt(event) {
    const targetEntity = event.hurtEntity;
	const damageSource = event.damageSource;
	const damagingEntity = damageSource === undefined ? undefined : damageSource.damagingEntity;
	const damagingEntityId = damagingEntity === undefined ? undefined : damagingEntity.typeId;
	const attackerMobHeadPassiveData = heads.getItemPassiveData(inventory.getHelmet(damagingEntity));
	const targetMobHeadPassiveData = heads.getItemPassiveData(inventory.getHelmet(targetEntity));

	if (targetEntity.typeId === "minecraft:wither")
	{
		// store dimension and loaction info on wither entity, to be used later for giving loot on death event
		game.host.setDynamicProperty("tcs_mbh_wither_dimension", targetEntity.dimension.id.replace("minecraft:",""));
		game.host.setDynamicProperty("tcs_mbh_wither_location", targetEntity.location);
	}

	// Handle attacker with mob head
	if (damagingEntity !== undefined)
	{
		if (damagingEntity.typeId === "tcs_mbh:spider_web")
			game.runCommand(`setblock ${Vector3.copy(targetEntity.location).toCommand()} web`);
		if (damagingEntity.typeId === "tcs_mbh:ender_dragon_fireball")
			damagingEntity.dimension.spawnParticle("tcs_mbh:ender_dragon_fireball_hit2", damagingEntity.location)

		//if (damagingEntity.typeId === "tcs_mbh:guardian_laser")
		//	damagingEntity.dimension.spawnParticle("tcs_mbh:guardian_laser_explosion", damagingEntity.location)

		if (attackerMobHeadPassiveData !== undefined) // attacker is player with mob head
		{
			if (attackerMobHeadPassiveData.hitEffect !== undefined)
				await updateHitEffect(damagingEntity, targetEntity, attackerMobHeadPassiveData.hitEffect);
			if (attackerMobHeadPassiveData.hitEffect2 !== undefined)
				await updateHitEffect(damagingEntity, targetEntity, attackerMobHeadPassiveData.hitEffect2);
			if (attackerMobHeadPassiveData.hitSound !== undefined)
				game.playSound(targetEntity, "@a", attackerMobHeadPassiveData.hitSound);
			if (attackerMobHeadPassiveData.id === "tcs_mbh:husk_head")
			{
				targetEntity.dimension.spawnParticle("tcs_mbh:husk_poison_fog", targetEntity.location);
				targetEntity.dimension.spawnParticle("tcs_mbh:husk_poison_skull", targetEntity.location);
			}

		}
	}

	if (targetMobHeadPassiveData !== undefined) // target is player with mob head
	{
		if (targetMobHeadPassiveData.effect !== undefined)
		{
			if (targetMobHeadPassiveData.effect.id === "tcs_mbh:reflect" && damagingEntity !== undefined)
			{
				let reflectDamage = event.damage * targetMobHeadPassiveData.effect.level / 100;
				damagingEntity.applyDamage(reflectDamage);
			}

			if (targetMobHeadPassiveData.effect.id == "tcs_mbh:fangs" && damageSource.cause === "entityAttack")
			{
				let fangs_rng = Math.floor(Math.random() * 100) + 1;
				if (fangs_rng <= 35)
				{
					game.runPlayerCommand(targetEntity, "function tcs/mbh/summon_fangs");
					/*let nerbyMobs = await game.getAllEntitiesInRange(targetEntity.dimension, targetEntity.location, 0.1, 3);
					for (let nearbyMob of nerbyMobs)
					{
						let loc = Vector3.copy(nearbyMob.location).toCommand();
						game.runPlayerCommand(targetEntity, `summon evocation_fang ${loc}`);
						//targetEntity.dimension.spawnEntity("evocation_fang", nearbyMob.location);
					}*/
				}
			}
		}

		if (targetMobHeadPassiveData.protect !== undefined)
		{
			if (targetMobHeadPassiveData.protect.id === "tcs_mbh:fall" && damageSource.cause === "fall")
			{
				const health = targetEntity.getComponent("health");
            	health.setCurrentValue(health.currentValue + event.damage);
			}
			if (targetMobHeadPassiveData.protect.id === "wither" && damageSource.cause === "wither")
			{
				const health = targetEntity.getComponent("health");
            	health.setCurrentValue(health.currentValue + event.damage);
			}
			if (targetMobHeadPassiveData.protect.id === "poison" && damageSource.cause === "poison")
			{
				const health = targetEntity.getComponent("health");
            	health.setCurrentValue(health.currentValue + event.damage);
			}
		}
	}
	
	let projectileSound = heads.getProjectileSound(damagingEntityId);
	if (projectileSound !== undefined)
		game.playSound(targetEntity, "@a", projectileSound);

	// Handle target with mob head
}

export async function onProjectileHitBlock(event) {
	let shooter = event.source;
	if (event.projectile.typeId === "tcs_mbh:spider_web")
	{
		game.runCommand(`execute positioned ${Vector3.copy(event.location).toCommand()} run fill ~-1 ~ ~ ~1 ~ ~ web replace air`);
		game.runCommand(`execute positioned ${Vector3.copy(event.location).toCommand()} run fill ~ ~-1 ~ ~ ~1 ~ web replace air`);
		game.runCommand(`execute positioned ${Vector3.copy(event.location).toCommand()} run fill ~ ~ ~-1 ~ ~ ~1 web replace air`);
		//game.runCommand(`setblock ${Vector3.copy(event.location).toCommand()} web`);
	}
	if (event.projectile.typeId === "tcs_mbh:tp_orb")
	{
		shooter.dimension.spawnParticle("tcs_mbh:enderman_glow1", shooter.location);
		shooter.dimension.spawnParticle("tcs_mbh:enderman_glow2", shooter.location);
		game.playSound(game.host, "@a", "tcs_mbh:enderman_teleport");
		//game.runPlayerCommand(shooter, `tp @s ${Vector3.copy(event.location).toCommand()}`);
		system.runTimeout(() => { delayedTeleport(shooter, event.location); }, 20);//!---
	}
	let projectileSound = heads.getProjectileSound(event.projectile.typeId);
	if (projectileSound !== undefined && event.projectile.typeId !== "tcs_mbh:guardian_laser")
		game.playSound(game.host, "@a", projectileSound);
	if (event.projectile.typeId === "tcs_mbh:ender_dragon_fireball")
		shooter.dimension.spawnParticle("tcs_mbh:ender_dragon_fireball_hit2", event.location);
	//if (event.projectile.typeId === "tcs_mbh:wither_skull")
	//	shooter.dimension.spawnParticle("tcs_mbh:wither_skull_hit2", event.location)
}
export async function delayedTeleport(player, tpLocation)
{
	await game.runPlayerCommand(player, `tp @s ${Vector3.copy(tpLocation).toCommand()}`);
	player.dimension.spawnParticle("tcs_mbh:enderman_glow1", tpLocation);
	player.dimension.spawnParticle("tcs_mbh:enderman_glow2", tpLocation);
	game.playSound(player, "@a", "tcs_mbh:enderman_teleport");
}

export async function onEffectAdd(event) {
    const targetEntity = event.entity;
	const mobHeadPassiveData = heads.getItemPassiveData(inventory.getHelmet(targetEntity));
	if (mobHeadPassiveData === undefined)
		return;

	if (mobHeadPassiveData.protect !== undefined)
		protectFromEffect(targetEntity, event.effect.typeId, mobHeadPassiveData.protect);
	if (mobHeadPassiveData.protect2 !== undefined)
		protectFromEffect(targetEntity, event.effect.typeId, mobHeadPassiveData.protect2);
	
}
async function protectFromEffect(player, effectGiven, protectData)
{

	if (protectData.inWater === true && !player.isInWater)
		return;

	if (protectData.id === effectGiven)
		player.removeEffect(effectGiven);
}

function print(coord)
{
	return "("+coord.x+" "+coord.y+" "+coord.z+")";
}

